# -*- coding: utf-8 -*-
# Generated by Django 1.11.24 on 2020-04-15 11:38
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("aimmo", "0012_auto_20200302_1310")]

    operations = [
        migrations.AlterField(
            model_name="game",
            name="can_play",
            field=models.ManyToManyField(
                help_text="List of auth_user IDs of users who are allowed to play and have access to the game.",
                related_name="playable_games",
                to=settings.AUTH_USER_MODEL,
            ),
        )
    ]
