# Generated by Django 2.2.24 on 2021-11-18 11:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("aimmo", "0025_generate_auth_token"),
    ]

    operations = [
        migrations.AlterField(
            model_name="game",
            name="auth_token",
            field=models.CharField(blank=True, max_length=48),
        ),  # Generated by Django, but doesn't actually change anything
        migrations.AlterField(
            model_name="game",
            name="worksheet",
            field=models.IntegerField(default=1),
        ),
        migrations.DeleteModel(
            name="Worksheet",
        ),
        migrations.RenameField(
            model_name="game",
            old_name="worksheet",
            new_name="worksheet_id",
        ),
    ]
